#include	<u.h>
#include	<stdarg.h>
#include	<libc.h>
#include	<stdio.h>
#include	<auth.h>
#include	<fcall.h>
#include	"db.h"

/* Write an index record.
 * _db_writedat() is called before this function, to set the fields
 * datoff and datlen in the DB structure, which we need to write
 * the index record. */

void
_db_writeidx(DB *db, const char *key,
	off_t offset, int whence, off_t ptrval)
{
	struct iovec	iov[2];
	char		asciiptrlen[PTR_SZ + IDXLEN_SZ +1];
	int		len;

	if ( (db->ptrval = ptrval) < 0 || ptrval > PTR_MAX)
		exits("invalid ptr");

	sprintf(db->idxbuf, "%s%c%d%c%d\n",
				key, SEP, db->datoff, SEP, db->datlen);
	if ( (len = strlen(db->idxbuf)) < IDXLEN_MIN || len > IDXLEN_MAX)
		exits("invalid length");
	sprintf(asciiptrlen, "%*d%*d", PTR_SZ, ptrval, IDXLEN_SZ, len);

		/* If we're appending, we have to lock before doing the seek()
		   and write() to make the two an atomic operation.  If we're
		   overwriting an existing record, we don't have to lock. */
	if (whence == SEEK_END)		/* we're appending */
		if (writew_lock(db->idxfd, ((db->nhash+1)*PTR_SZ)+1,
							SEEK_SET, 0) < 0)
			exits("writew_lock error");

		/* Position the index file and record the offset */
	if ( (db->idxoff = seek(db->idxfd, offset, whence)) == -1)
		exits("seek error");

	iov[0].iov_base = asciiptrlen;
	iov[0].iov_len  = PTR_SZ + IDXLEN_SZ;
	iov[1].iov_base = db->idxbuf;
	iov[1].iov_len  = len;
	if (writev(db->idxfd, &iov[0], 2) != PTR_SZ + IDXLEN_SZ + len)
		exits("writev error of index record");

	if (whence == SEEK_END)
		if (un_lock(db->idxfd, ((db->nhash+1)*PTR_SZ)+1, SEEK_SET, 0) < 0)
			exits("un_lock error");
}

int
writew_lock(int fd, int offset, int whence, int len) {
USED(fd);
USED(offset);
USED(whence);
USED(len);
return 1;
}

int
readw_lock(int fd, int offset, int whence, int len) {
USED(fd);
USED(offset);
USED(whence);
USED(len);
return 1;
}

int
un_lock(int fd, int offset, int whence, int len) {
USED(fd);
USED(offset);
USED(whence);
USED(len);
return 1;
}

int
writev(int fd, struct iovec *iov, int count) {
	char *buff, *bp;
	uint i, j, jj, bytes;
	long written;

	bytes = 0;
	for (i = 0; i < count; ++i)
		bytes += iov[i].iov_len;
	if (bytes == 0)
		return 0;
	buff = (char*)malloc(bytes);

	j = bytes;
	bp = buff;
	for (i = 0; i < count; ++i) {
		jj = min(iov[i].iov_len, j);
		memcpy((void*)bp, iov[i].iov_base, (long)jj);
		bp += jj;
		j -= jj;
		if (bytes == 0)
			break;
	}
	written = write(fd, (void*)buff, (long)bytes);
	free(buff);
	return (int)written;
}
