#include "lib.h"
#include <ctype.h>
#include <db.h>
#include "trie.h"
#include "darray.h"
#include "phones.h"

Trie*	phones;

static void 
enter_phones(void)
{
	int	i;
	char	*s;
	for(i = 1; (s = ph_name[i]); i++)
		trie_insert(&phones, s, (void * ) i);
}

static void
enter_words(DB *db, Biobuf *f)
{
	char	*buf, *s, *s1, *h, key[128];

	while((buf = Brdline(f, '\n')) != nil) {
		buf[Blinelen(f)-1] = 0;
		s = buf;
		h = strchr(s, '#');
		if(h)
			*h = '\0';
		while(isspace(*s))
			s++;
		if(*s) {
			char	*p = s;
			while(isalpha(*p) || *p == '-' || *p == '\'' ) {
/*
			while(isalpha(*p) || *p == '-' || *p == '&' || *p == '%' ||
			*p == '\\' || *p == '\'' || *p == '"' || *p=='!') {
*/
				if(islower(*p))
					*p = toupper(*p);
				p++;
			}
			if(isspace(*p)) {
				char	codes[4096];
				char	*d = codes;
				int	i, ok = 1;
				s1 = s;
				for(i=0;i<(p-s);i++) key[i] = *s1++;
				key[i] = 0;
				while(*p && ok) {
					unsigned	code;
					while(isspace(*p))
						p++;
					if(*p) {	/* phonetic codes */
						char	*e = p;
						while(isalpha(*e) || *e == '1' || *e == '2') {
							if(islower(*e))
								*e = toupper(*e);
							e++;
						}
						if(*e == '0')
							*e++ = ' ';
						if(e > p && (code = (unsigned) trie_lookup(&phones, &p)))
							*d++ = code;
						else {
							fprint(2, "Bad code %.*s>%s", (int)(p - s), s, p);
							ok = 0;
							break;
						}
					}
/* print("\ncode = %X", *(d-1));	 K.Okamoto for debug */
				}
				if(ok) {
					*d = 0;
/*  for debug  K.Okamoto					
print("\nkey = %s", key);
print("\ndata = %s", codes);
*/
					db_store(db, key, codes, DB_INSERT);
				}
			}else {
				if(*p != '(')
					fprint(2, "Ignore (%c) %s", *p, s);
			}
		}
	}
}

void
main(int argc, char **argv)
{
	Biobuf *f;
	DB *db;

	if(argc != 3) {
		fprint(2, "Usage: mkdictdb dict out.db\n");
		exits("usage");
	}
	f = Bopen(argv[1], OREAD);
	if(f == nil) {
		fprint(2, "mkdictdb: can't open %s: %r\n", argv[1]);
		exits("open");
	}
	db = db_open(argv[2], O_RDWR|O_CREAT, 00666);
	if(db == nil) {
		fprint(2, "mkdictdb: can't create %s: %r\n", argv[2]);
		exits("create");
	}
	enter_phones();
	enter_words(db, f);
	db_close(db);
	Bterm(f);
	exits(nil);
}
